# 代码规范

[toc]

## 一、编程规约

### （一）命名风格

1. **变量命名规约：**

   **<u>1.1 不以下划线(_)或者美元符号($)开头或结尾。</u>**

      反例： _name / __name / $Object / name_ / name$ / Object$

   <u>**1.2 采用全英文命名，不使用中英文混合或者纯中文、对于已经国际化的单词如hangzhou可以直接使用。杜绝完全不规范的缩写，避免望文不知义。**</u>

   > [通用缩写规范示例](https://blog.csdn.net/zhaoguanghui2012/article/details/51144156)

   ​    正例： alibaba / taobao / youku / hangzhou 等国际通用的名称，可视同英文。

   ​	反例： DaZhePromotion [ 打折 ] / getPingfenByName() [ 评分 ] / int 某变量 = 3

   <u>**1.3 变量、方法名、成员变量、局部变量都统一采用 lowerCamelCase 风格，必须遵从驼峰形式。**</u>

   >  lowerCamelCase 风格  
   >
   > 它的名字是由多个和在一起的词组合成的一个单词且它们的每个词（除了第一个词以外）的第一个字母大写。

    正例： localValue / getHttpMessage() / inputUserId

   <u>**1.4  常量的命名全部大写、单词间用下划线分割**</u>

   ​	正例： MAX _ STOCK _ COUNT
   ​	反例： MAX _ COUNT

   <u>**1.5 数组定义：中括号是数组类型的一部分，数组定义如下： String[] args;**</u>
   	反例：使用 String args[] 的方式来定义。

   <u>**1.6 POJO 类中布尔类型的变量，都不要加 is ，否则部分框架解析会引起序列化错误。**</u>
   	反例： 定义为基本数据类型 Boolean isDeleted； 的属性，它的方法也是 isDeleted() ， RPC框	架在反向解析的时候，“以为”对应的属性名称是 deleted ，导致属性获取不到，进而抛出异
   	常。

2. **类名/接口/方法/包命名规约：**

   **<u>2.1 类名使用 UpperCamelCase 风格，必须遵从驼峰形式，但以下情形例外： DO / PO/BO /DTO / VO / AO/DAO</u>**

   > UpperCamelCase风格：
   >
   >  第一个词的首字母，以及后面每个词的首字母都大写
   >
   > DO（ Data Object）与数据库表结构一一对应，通过DAO层向上传输数据源对象。
   >
   > PO(Persistant Object) 持久对象
   >
   > BO(Business Object) 业务对象
   >
   > DTO(Data <u>**Transfer**</u> Object) 数据传输对象
   >
   > VO(Value Object) 表现对象
   >
   > AO（ Application Object）
   >
   > DAO(Data access object) 数据访问对象
   >
   > POJO(Plain ordinary java object) 简单java对象,POJO是DO/DTO/BO/VO的统称，**禁止命名成xxxPOJO**。
   >
   > [上述对象简介资料](https://blog.csdn.net/yu1014745867/article/details/89218322)

   正例： MarcoPolo / UserDO / XmlService / TcpUdpDeal / TaPromotion
   反例： macroPolo / UserDo / XMLService / TCPUDPDeal / TAPromotion

   **<u>2.2 抽象类命名使用 Abstract 或 Base 开头 ； 异常类命名使用 Exception 结尾 ； 测试类命名以它要测试的类的名称开始，以 Test 结尾。</u>**

   <u>**2.3 如果模块、接口、类、方法使用了设计模式，在命名时体现出具体模式。**</u>
   	正例： public class OrderFactory;
   				public class LoginProxy;
   				public class ResourceObserver;

   **<u>2.4 接口类中的方法和属性不要加任何修饰符号 （public 也不要加 ） ，保持代码的简洁性，并加上有效的 Javadoc 注释。尽量不要在接口里定义变量，如果一定要定义变量，肯定是与接口方法相关，并且是整个应用的基础常量。</u>**

   > [Javadoc 注释](https://www.runoob.com/java/java-documentation.html)

   ​	正例：接口方法签名： void f();
   ​	接口基础常量表示： String COMPANY = " alibaba " ;
   ​	反例：接口方法定义： public abstract void f();

   **<u>2.5 接口和实现类的命名有两套规则：</u>**
   对于 Service 和 DAO 类，基于 SOA 的理念，暴露出来的服务一定是接口，内部
   的实现类用 Impl 的后缀与接口区别。
   正例： CacheServiceImpl 实现 CacheService 接口。
    如果是形容能力的接口名称，取对应的形容词做接口名 （ 通常是– able 的形式 ） 。
   正例： AbstractTranslator 实现 Translatable 。

   **<u>2.6 枚举类名建议带上 Enum 后缀，枚举成员名称需要全大写，单词间用下划线隔开。</u>**

   >  说明：枚举其实就是特殊的常量类，且构造方法被默认强制是私有。

   正例：枚举名字为 ProcessStatusEnum 的 成员名称： SUCCESS / UNKOWN _ REASON 。

   **<u>2.7 各层命名规约：</u>**
   	<u>A) Service / DAO 层方法命名规约</u>
   	1 ） 获取单个对象的方法用 get 做前缀。
   	2 ） 获取多个对象的方法用 list 做前缀。
   	3 ） 获取统计值的方法用 count 做前缀。
   	4 ） 插入的方法用 save/insert 做前缀。
   	5 ） 删除的方法用 remove/delete 做前缀。
   	6 ） 修改的方法用 update 做前缀。
   	<u>B) 领域模型命名规约</u>
   	1 ） 数据对象： xxxDO ， xxx 即为数据表名。
   	2 ） 数据传输对象： xxxDTO ， xxx 为业务领域相关的名称。
   	3 ） 展示对象： xxxVO ， xxx 一般为网页名称。
   	4 ） POJO 是 DO / DTO / BO / VO 的统称，禁止命名成 xxxPOJO 。

   **<u>2.8  包名统一使用小写，点分隔符之间有且仅有一个自然语义的英语单词。包名统一使用单数形式，但是类名如果有复数含义，类名可以使用复数形式。</u>***
   	正例： 应用工具类包名为 com . alibaba . open . util 、类名为 MessageUtils（ 此规则参考
   	spring 的框架结构 ）

### （二）常量定义

1. **<u>常量必须预先定义过才能使用。</u>**

2. **<u>long或者Long初始赋值时、使用大写L代替小写l，防止与数字1混淆。</u>**

3. **<u>[推荐]不需要使用一个常量类维护所有变量。按照常量功能进行归类。</u>**

   正例：缓存相关常量放在类 CacheConsts 下 ； 系统配置相关常量放在类 ConfigConsts 下。

4. **<u>[推荐]常量的复用层次有五层：跨应用共享常量、应用内共享常量、子工程内共享常量、包内共享常量、类内共享常量。</u>**
   1 ） 跨应用共享常量：放置在二方库中，通常是 client . jar 中的 constant 目录下。

   2 ） 应用内共享常量：放置在一方库中，通常是 modules 中的 constant 目录下。

   3 ） 子工程内部共享常量：即在当前子工程的 constant 目录下。
   4 ） 包内共享常量：即在当前包下单独的 constant 目录下。
   5 ） 类内共享常量：直接在类内部 private static final 定义。

5. **<u>[推荐]如果变量值仅在一个范围内变化，且带有名称之外的延伸属性，定义为枚举类。</u>**

   下面正例中的数字就是延伸信息，表示星期几。
   正例： public Enum { MONDAY(1), TUESDAY(2), WEDNESDAY(3), THURSDAY(4), FRIDAY(5), SATURDAY(6),
   SUNDAY(7);}

### （三）代码格式

1. **<u>大括号的使用约定。如果是大括号内为空，则简洁地写成{}即可，不需要换行 ； 如果是非空代码块则：</u>**
   1 ） 左大括号前不换行。
   2 ） 左大括号后换行。
   3 ） 右大括号前换行。
   4 ） 右大括号后还有 else 等代码则不换行 ； 表示终止的右大括号后必须换行。

2. **<u>左小括号和字符之间不出现空格 ； 同样，右小括号和字符之间也不出现空格。</u>**
   反例： if (空格 a == b 空格)

3.  **<u>if / for / while / switch / do 等保留字与括号之间都必须加空格。</u>**

4. **<u>任何二目、三目运算符的左右两边都需要加一个空格。</u>**
   说明：运算符包括赋值运算符=、逻辑运算符&&、加减乘除符号等。

5. <u>**采用 4 个空格缩进，禁止使用 tab 字符。**</u>

   >如果使用 tab 缩进，必须设置 1 个 tab 为 4 个空格。IDEA 设置 tab 为 4 个空格时，
   >请勿勾选 Use tab character ；而在 eclipse 中，必须勾选 insert spaces for tabs 。

6. <u>**注释的双斜线与注释内容之间有且仅有一个空格。**</u>
   正例： // 注释内容，注意在 // 和注释内容之间有一个空格。

7. **<u>单行字符数限制不超过 120 个，超出需要换行，换行时遵循如下原则：</u>**
   1） 第二行相对第一行缩进 4 个空格，从第三行开始，不再继续缩进，参考示例。
   2 ） 运算符与下文一起换行。
   3 ） 方法调用的点符号与下文一起换行。
   4 ） 方法调用时，多个参数，需要换行时，在逗号后进行。
   5 ） 在括号前不要换行，见反例。
   正例：
   StringBuffer sb = new StringBuffer();
   // 超过 120 个字符的情况下，换行缩进 4 个空格，点号和方法名称一起换行
   sb.append("zi").append("xin")...
   .append("huang")...
   .append("huang")...
   .append("huang");
   反例：
   StringBuffer sb = new StringBuffer();
   // 超过 120 个字符的情况下，不要在括号前换行
   sb.append("zi").append("xin")...append
   ("huang");
   // 参数很多的方法调用可能超过 120 个字符，不要在逗号前换行
   method(args1, args2, args3, ...
   , argsX);

8. **<u>方法参数在定义和传入时，多个参数逗号后边必须加空格。</u>**
   正例：下例中实参的" a ",后边必须要有一个空格。
   method("a", "b", "c");

9.  <u>**IDE 的 text file encoding 设置为 UTF -8 ; IDE 中文件的换行符使用 Unix 格式，不要使用 Windows 格式。**</u>

10. **<u>[推荐]没有必要增加若干空格来使某一行的字符与上一行对应位置的字符对齐。</u>**
    正例：
    int a = 3;
    long b = 4L;
    float c = 5F;
    StringBuffer sb = new StringBuffer();

11. **<u>[推荐]方法体内的执行语句组、变量的定义语句组、不同的业务逻辑之间或者不同的语义之间插入一个空行。相同业务逻辑和语义之间不需要插入空行。</u>**
    说明：没有必要插入多个空行进行隔开。

### （四）OOP规约

1. **<u>避免通过一个类的对象引用访问此类的静态变量或静态方法，无谓增加编译器解析成本，直接用类名来访问即可。</u>**

2. **<u>所有的覆写方法，必须加@ Override 注解。</u>**

   >  getObject() 与 get 0 bject() 的问题。一个是字母的 O ，一个是数字的 0，加@ Override可以准确判断是否覆盖成功。另外，如果在抽象类中对方法签名进行修改，其实现类会马上编译报错。

3. **<u>外部正在调用或者二方库依赖的接口，不允许修改方法签名，避免对接口调用方产生影响。接口过时必须加@ Deprecated 注解，并清晰地说明采用的新接口或者新服务是什么。</u>**

4. **<u>不能使用过时的类或方法。</u>**

5. <u>**Object 的 equals 方法容易抛空指针异常，应使用常量或确定有值的对象来调用equals**</u> 

   正例：" test " .equals(object);
   反例： object.equals( " test " );

   说明：推荐使用 java . util . Objects # equals（JDK 7 引入的工具类 ）

6. **<u>相同参数类型，相同业务含义，才可以使用 Java 的可变参数，避免使用 Object 。</u>**
   说明：可变参数必须放置在参数列表的最后。 （ 提倡同学们尽量不用可变参数编程 ）
   正例： public User getUsers(String type, Integer... ids) {...}

7. **<u>所有的相同类型的包装类对象之间值的比较，全部使用 equals 方法比较。</u>**
   说明：对于 Integer var = ? 在-128 至 127 范围内的赋值， Integer 对象是在IntegerCache . cache 产生，会复用已有对象，这个区间内的 Integer 值可以直接使用==进行判断，但是这个区间之外的所有数据，都会在堆上产生，并不会复用已有对象，这是一个大坑，推荐使用 equals 方法进行判断。

8. **<u>关于基本数据类型与包装数据类型的使用标准如下：</u>**

   1 ） 【强制】所有的 POJO 类属性必须使用包装数据类型。
   2 ） 【强制】 RPC 方法的返回值和参数必须使用包装数据类型。
   3 ） 【推荐】所有的局部变量使用基本数据类型。

   说明： POJO 类属性没有初值是提醒使用者在需要使用时，必须自己显式地进行赋值，任何NPE 问题，或者入库检查，都由使用者来保证。
   正例：数据库的查询结果可能是 null ，因为自动拆箱，用基本数据类型接收有 NPE 风险。
   反例：比如显示成交总额涨跌情况，即正负 x %， x 为基本数据类型，调用的 RPC 服务，调用不成功时，返回的是默认值，页面显示为 0%，这是不合理的，应该显示成中划线。所以包装数据类型的 null 值，能够表示额外的信息，如：远程调用失败，异常退出。

9. **<u>定义 DO / DTO / VO 等 POJO 类时，不要设定任何属性默认值。</u>**
   反例： POJO 类的 gmtCreate 默认值为 new Date(); 但是这个属性在数据提取时并没有置入具
   体值，在更新其它字段时又附带更新了此字段，导致创建时间被修改成当前时间。

10. **<u>序列化类新增属性时，请不要修改 serialVersionUID 字段，避免反序列失败 ； 如果完全不兼容升级，避免反序列化混乱，那么请修改 serialVersionUID 值。</u>**
    说明：注意 serialVersionUID 不一致会抛出序列化运行时异常。

11. <u>**构造方法里面禁止加入任何业务逻辑，如果有初始化逻辑，请放在 init 方法中。**</u>

12. **<u>POJO 类必须写 toString 方法。使用 IDE 的中工具： source > generate toString 时，如果继承了另一个 POJO 类，注意在前面加一下 super . toString 。</u>**

    说明：在方法执行抛出异常时，可以直接调用 POJO 的 toString() 方法打印其属性值，便于排
    查问题。

13. **<u>[推荐]使用索引访问用 String 的 split 方法得到的数组时，需做最后一个分隔符后有无内容的检查，否则会有抛 IndexOutOfBoundsException 的风险。</u>**

    说明：
    String str = "a,b,c,,";
    String[] ary = str.split(",");
    // 预期大于 3，结果是 3
    System.out.println(ary.length);

14. **<u>当一个类有多个构造方法，或者多个同名方法，这些方法应该按顺序放置在一起，便于阅读</u>**。

15. **<u>[推荐]类内方法定义顺序依次是：公有方法或保护方法 > 私有方法 > getter / setter方法。</u>**

16. **<u>[推荐]setter 方法中，参数名称与类成员变量名称一致， this .成员名 = 参数名。在getter / setter 方法中，不要增加业务逻辑，增加排查问题的难度。</u>**

    反例：
    public Integer getData() {
    if (true) {
    return this.data + 100;
    } else {
    return this.data - 100;
    }
    }

17. **<u>[推荐]循环体内，字符串的连接方式，使用 StringBuilder 的 append 方法进行扩展。</u>**
    说明：反编译出的字节码文件显示每次循环都会 new 出一个 StringBuilder 对象，然后进行
    append 操作，最后通过 toString 方法返回 String 对象，造成内存资源浪费。
    反例：
    String str = "start";
    for (int i = 0; i < 100; i++) {
    str = str + "hello";
    }

18.  **<u>[推荐]final 可以声明类、成员变量、方法、以及本地变量，下列情况使用 final 关键字：</u>**
    1） 不允许被继承的类，如： String 类。
    2） 不允许修改引用的域对象，如： POJO 类的域变量。
    3） 不允许被重写的方法，如： POJO 类的 setter 方法。
    4） 不允许运行过程中重新赋值的局部变量。
    5） 避免上下文重复使用一个变量，使用 final 描述可以强制重新定义一个变量，方便更好地进行重构。

19. **<u>[推荐]慎用 Object 的 clone 方法来拷贝对象。</u>**
    说明：对象的 clone 方法默认是浅拷贝，若想实现深拷贝需要重写 clone 方法实现属性对象
    的拷贝。

20. **<u>[推荐]类成员与方法访问控制从严：</u>**
    1 ） 如果不允许外部直接通过 new 来创建对象，那么构造方法必须是 private 。
    2 ） 工具类不允许有 public 或 default 构造方法。
    3 ） 类非 static 成员变量并且与子类共享，必须是 protected 。
    4 ） 类非 static 成员变量并且仅在本类使用，必须是 private 。
    5 ） 类 static 成员变量如果仅在本类使用，必须是 private 。

    6 ） 若是 static 成员变量，必须考虑是否为 final 。
    7 ） 类成员方法只供类内部调用，必须是 private 。
    8 ） 类成员方法只对继承类公开，那么限制为 protected 。
    说明：任何类、方法、参数、变量，严控访问范围。过于宽泛的访问范围，不利于模块解耦。
    思考：如果是一个 private 的方法，想删除就删除，可是一个 public 的 service 方法，或者
    一个 public 的成员变量，删除一下，不得手心冒点汗吗？变量像自己的小孩，尽量在自己的
    视线内，变量作用域太大，无限制的到处跑，那么你会担心的。

### （五）集合处理

1. **<u>关于 hashCode 和 equals 的处理，遵循如下规则：</u>**
1） 只要重写 equals ，就必须重写 hashCode 。
2） 因为 Set 存储的是不重复的对象，依据 hashCode 和 equals 进行判断，所以 Set 存储的对象必须重写这两个方法。
3） 如果自定义对象做为 Map 的键，那么必须重写 hashCode 和 equals 。
说明： String 重写了 hashCode 和 equals 方法，所以我们可以非常愉快地使用 String 对象作为 key 来使用。

2. **<u>ArrayList 的 subList 结果不可强转成 ArrayList ，否则会抛出 ClassCastException</u>**
  异常，即 java . util . RandomAccessSubList cannot be cast to java . util . ArrayList .

  说明： subList 返回的是 ArrayList 的内部类 SubList ，并不是 ArrayList ，而是
  ArrayList 的一个**视图**，对于 SubList 子列表的所有操作最终会反映到原列表上。

3. **<u>在 subList 场景中，高度注意对原集合元素个数的修改，会导致子列表的遍历、增加、删除均会产生 ConcurrentModificationException 异常。</u>**

4. **<u>使用集合转数组的方法，必须使用集合的 toArray(T[] array) ，传入的是类型完全一样的数组，大小就是 list . size() 。</u>**
  说明：使用 toArray 带参方法，入参分配的数组空间不够大时， toArray 方法内部将重新分配内存空间，并返回新数组地址 ； 如果数组元素大于实际所需，下标为 [ list . size() ] 的数组元素将被置为 null ，其它数组元素保持原值，因此最好将方法入参数组大小定义与集合元素个数一致。
  正例：
  List<String> list = new ArrayList<String>(2);
  list.add("guan");
  list.add("bao");
  String[] array = new String[list.size()];
  array = list.toArray(array);

5. **<u>使用工具类 Arrays . asList() 把数组转换成集合时，不能使用其修改集合相关的方法，它的 add / remove / clear 方法会抛出 UnsupportedOperationException 异常。</u>**
  说明： asList 的返回对象是一个 Arrays 内部类，并没有实现集合的修改方法。 Arrays . asList体现的是适配器模式，只是转换接口，后台的数据仍是数组。
  String[] str = new String[] { "you", "wu" };
  List list = Arrays.asList(str);
  第一种情况： list.add("yangguanbao"); 运行时异常。
  第二种情况： str[0] = "gujin"; 那么 list.get(0) 也会随之修改。

6. **<u>泛型通配符<? extends T >来接收返回的数据，此写法的泛型集合不能使用 add 方法，而 <? super T> 不能使用 get 方法，做为接口调用赋值时易出错。</u>**
  说明：扩展说一下 PECS(Producer Extends Consumer Super) 原则：第一、频繁往外读取内
  容的，适合用<? extends T >。第二、经常往里插入的，适合用 <? super T> 。

7. **<u>不要在 foreach 循环里进行元素的 remove / add 操作。 remove 元素请使用 Iterator方式，如果并发操作，需要对 Iterator 对象加锁。</u>**
  正例：
  Iterator<String> iterator = list.iterator();
  while (iterator.hasNext()) {
  String item = iterator.next();
  if (删除元素的条件) {
  iterator.remove();
  }
  }
  反例：
  List<String> list = new ArrayList<String>();
  list.add("1");
  list.add("2");
  for (String item : list) {
  if ("1".equals(item)) {
  list.remove(item);
  }
  }
  说明：以上代码的执行结果肯定会出乎大家的意料，那么试一下把“1”换成“2”，会是同样的结果吗？

8. **<u>在 JDK 7 版本及以上， Comparator 要满足如下三个条件，不然 Arrays . sort ，Collections . sort 会报 IllegalArgumentException 异常。</u>**
  说明：三个条件如下
  1 ） x ， y 的比较结果和 y ， x 的比较结果相反。

  2 ） x > y ， y > z ，则 x > z 。
  3 ） x = y ，则 x ， z 比较结果和 y ， z 比较结果相同。
  反例：下例中没有处理相等的情况，实际使用中可能会出现异常：
  new Comparator<Student>() {
  @Override
  public int compare(Student o1, Student o2) {
  return o1.getId() > o2.getId() ? 1 : -1;
  }
  };

9. **<u>[推荐]集合初始化时，指定集合初始值大小。</u>**
   说明： HashMap 使用 HashMap(int initialCapacity) 初始化，
   正例： initialCapacity = (需要存储的元素个数 / 负载因子) + 1。注意负载因子（即loader factor）默认为 0.75， 如果暂时无法确定初始值大小，请设置为 16（即默认值）。
   反例： HashMap 需要放置 1024 个元素，由于没有设置容量初始大小，随着元素不断增加，容量 7 次被迫扩大， resize 需要重建 hash 表，严重影响性能。

10. **<u>[推荐]使用 entrySet 遍历 Map 类集合 KV ，而不是 keySet 方式进行遍历。</u>**
说明： keySet 其实是遍历了 2 次，一次是转为 Iterator 对象，另一次是从 hashMap 中取出key 所对应的 value 。而 entrySet 只是遍历了一次就把 key 和 value 都放到了 entry 中，效率更高。如果是 JDK 8，使用 Map . foreach 方法。
正例： values() 返回的是 V 值集合，是一个 list 集合对象 ；keySet() 返回的是 K 值集合，是一个 Set 集合对象 ；entrySet() 返回的是 K - V 值组合集合。

11. **<u>[推荐]高度注意 Map 类集合 K / V 能不能存储 null 值的情况，如下表格：</u>**

    | 集合类            | Key           | Value         | Super       | 说明                   |
    | ----------------- | ------------- | ------------- | ----------- | ---------------------- |
    | Hashtable         | 不允许为 null | 不允许为 null | Dictionary  | 线程安全               |
    | ConcurrentHashMap | 不允许为 null | 不允许为 null | AbstractMap | 锁分段技术（JDK8:CAS） |
    | TreeMap           | 不允许为 null | 允许为 null   | AbstractMap | 线程不安全             |
    | HashMap           | 允许为 null   | 允许为 null   | AbstractMap | 线程不安全             |

    反例： 由于 HashMap 的干扰，很多人认为 ConcurrentHashMap 是可以置入 null 值，而事实上，
    存储 null 值时会抛出 NPE 异常。

12. **<u>[推荐]合理利用好集合的有序性 (sort) 和稳定性 (order) ，避免集合的无序性 (unsort) 和不稳定性 (unorder) 带来的负面影响。</u>**
    说明：有序性是指遍历的结果是按某种比较规则依次排列的。稳定性指集合每次遍历的元素次序是一定的。如： ArrayList 是 order / unsort；HashMap 是 unorder / unsort；TreeSet 是order / sort 。

13. **<u>利用 Set 元素唯一的特性，可以快速对一个集合进行去重操作，避免使用 List 的contains 方法进行遍历、对比、去重操作。</u>**

### （六）控制语句

1. **<u>在一个 switch 块内，每个 case 要么通过 break / return 等来终止，要么注释说明程序将继续执行到哪一个 case 为止 ； 在一个 switch 块内，都必须包含一个 default 语句并且放在最后，即使它什么代码也没有。</u>***
2. **<u>在 if / else / for / while / do 语句中必须使用大括号。即使只有一行代码，避免采用单行的编码方式： if (condition) statements;</u>**
3. **<u>[推荐]表达异常的分支时，少用 if-else 方式 ，这种方式可以改写成：</u>**
if (condition) {
...
return obj;
}
// 接着写 else 的业务逻辑代码;
说明：如果非得使用 if()...else if()...else... 方式表达逻辑，【强制】避免后续代码维护困难，请勿超过 3 层。
正例：超过 3 层的 if-else 的逻辑判断代码可以使用卫语句、策略模式、状态模式等来实现，其中卫语句示例如下：
public void today() {
if (isBusy()) {
System.out.println(“change time.”);
return;
}
if (isFree()) {
System.out.println(“go to travel.”);
return;
}
4. **<u>[推荐]除常用方法（如 getXxx/isXxx）等外，不要在条件判断中执行其它复杂的语句，将复杂逻辑判断的结果赋值给一个有意义的布尔变量名，以提高可读性。</u>**
  说明：很多 if 语句内的逻辑相当复杂，阅读者需要分析条件表达式的最终结果，才能明确什么
  样的条件执行什么样的语句，那么，如果阅读者分析逻辑表达式错误呢？
  正例：
  // 伪代码如下
  final boolean existed = (file.open(fileName, "w") != null) && (...) || (...);
  if (existed) {
  ...
  }
  反例：
  if ((file.open(fileName, "w") != null) && (...) || (...)) {
  ...
  }

5. **<u>[推荐]循环体中的语句要考量性能，以下操作尽量移至循环体外处理，如定义对象、变量、获取数据库连接，进行不必要的 try - catch 操作 （ 这个 try - catch 是否可以移至循环体外 ） 。</u>**
6. **<u>[推荐]接口入参保护，这种场景常见的是用于做批量操作的接口。</u>**
7. **<u>下列情形，需要进行参数校验：</u>**
1 ） 调用频次低的方法。
2 ） 执行时间开销很大的方法。此情形中，参数校验时间几乎可以忽略不计，但如果因为参
数错误导致中间执行回退，或者错误，那得不偿失。
3 ） 需要极高稳定性和可用性的方法。
4 ） 对外提供的开放接口，不管是 RPC / API / HTTP 接口。
5） 敏感权限入口。
8. **<u>下列情形，不需要进行参数校验：</u>**
1 ） 极有可能被循环调用的方法。但在方法说明里必须注明外部参数检查要求。
2 ） 底层调用频度比较高的方法。毕竟是像纯净水过滤的最后一道，参数错误不太可能到底层才会暴露问题。一般 DAO 层与 Service 层都在同一个应用中，部署在同一台服务器中，所以 DAO 的参数校验，可以省略。
3 ） 被声明成 private 只会被自己代码所调用的方法，如果能够确定调用方法的代码传入参数已经做过检查或者肯定不会有问题，此时可以不校验参数。

### （七）注释规约

1. <u>**类、类属性、类方法的注释必须使用 Javadoc 规范，使用(/**内容*/)格式，不得使用// xxx 方式。</u>
说明：在 IDE 编辑窗口中， Javadoc 方式会提示相关注释，生成 Javadoc 可以正确输出相应注释 ； 在 IDE 中，工程调用方法时，不进入方法即可悬浮提示方法、参数、返回值的意义，提高阅读效率。

2. **<u>所有的抽象方法 （ 包括接口中的方法 ） 必须要用 Javadoc 注释、除了返回值、参数、异常说明外，还必须指出该方法做什么事情，实现什么功能。</u>**
  说明：对子类的实现要求，或者调用注意事项，请一并说明。

3. **<u>所有的类都必须添加创建者和创建日期。</u>**

4. **<u>方法内部单行注释，在被注释语句上方另起一行，使用//注释。方法内部多行注释使用/* */注释，注意与代码对齐。</u>**

5. **<u>所有的枚举类型字段必须要有注释，说明每个数据项的用途。</u>**

6. **<u>[推荐]与其“半吊子”英文来注释，不如用中文注释把问题说清楚。专有名词与关键字保持英文原文即可。</u>**
  反例：“ TCP 连接超时”解释成“传输控制协议连接超时”，理解反而费脑筋。

7. **<u>[推荐]代码修改的同时，注释也要进行相应的修改，尤其是参数、返回值、异常、核心逻辑等的修改。</u>**
  说明：代码与注释更新不同步，就像路网与导航软件更新不同步一样，如果导航软件严重滞后，就失去了导航的意义。

8. **<u>谨慎注释掉代码。在上方详细说明，而不是简单地注释掉。如果无用，则删除。</u>***
  说明：代码被注释掉有两种可能性：1 ） 后续会恢复此段代码逻辑。2 ） 永久不用。前者如果没有备注信息，难以知晓注释动机。后者建议直接删掉 （ 代码仓库保存了历史代码 ） 。

9. **<u>对于注释的要求：</u>**

  第一、能够准确反应设计思想和代码逻辑 ；

   第二、能够描述业务含义，使别的程序员能够迅速了解到代码背后的信息。完全没有注释的大段代码对于阅读者形同天书，注释是给自己看的，即使隔很长时间，也能清晰理解当时的思路 ； 注释也是给继任者看的，使其能够快速接替自己的工作。

10. **<u>好的命名、代码结构是自解释的，注释力求精简准确、表达到位。避免出现注释的一个极端：过多过滥的注释，代码的逻辑一旦修改，修改注释是相当大的负担。</u>**
    反例：
    // put elephant into fridge
    put(elephant, fridge);

    方法名 put ，加上两个有意义的变量名 elephant 和 fridge ，已经说明了这是在干什么，语义清晰的代码不需要额外的注释。

11. **<u>特殊注释标记，请注明标记人与标记时间。</u>**

    注意及时处理这些标记，通过标记扫描，经常清理此类标记。线上故障有时候就是来源于这些标记处的代码。
    1 ） 待办事宜 （TODO） : （ 标记人，标记时间， [ 预计处理时间 ]）
    表示需要实现，但目前还未实现的功能。这实际上是一个 Javadoc 的标签，目前的 Javadoc还没有实现，但已经被广泛使用。只能应用于类，接口和方法 （ 因为它是一个 Javadoc 标签 ） 。
    2 ） 错误，不能工作 （FIXME） : （ 标记人，标记时间， [ 预计处理时间 ]）
    在注释中用 FIXME 标记某代码是错误的，而且不能工作，需要及时纠正的情况。

### （八）其他

1. **<u>在使用正则表达式时，利用好其预编译功能，可以有效加快正则匹配速度。</u>**
  说明：不要在方法体内定义： Pattern pattern = Pattern . compile( 规则 );

2.  **<u>velocity 调用 POJO 类的属性时，建议直接使用属性名取值即可</u>**，模板引擎会自动按规范调用 POJO 的 getXxx() ，如果是 boolean 基本数据类型变量 （boolean 命名不需要加 is前缀 ） ，会自动调用 isXxx() 方法。
说明：注意如果是 Boolean 包装类对象，优先调用 getXxx() 的方法。

3. **<u>后台输送给页面的变量必须加 $!{var} ——中间的感叹号。</u>**
  说明：如果 var = null 或者不存在，那么 ${var} 会直接显示在页面上。

4. **<u>注意 Math . random() 这个方法返回是 double 类型，注意取值的范围 0≤ x <1 （ 能够取到零值，注意除零异常 ） ，如果想获取整数类型的随机数，不要将 x 放大 10 的若干倍然后取整，直接使用 Random 对象的 nextInt 或者 nextLong 方法。</u>**

5. **<u>获取当前毫秒数 System . currentTimeMillis(); 而不是 new Date() . getTime();</u>**
说明：如果想获取更加精确的纳秒级时间值，使用 System . nanoTime()的方式 。在 JDK 8 中，针对统计时间等场景，推荐使用 Instant 类。

6. **<u>不要在视图模板中加入任何复杂的逻辑。</u>**
  说明：根据 MVC 理论，视图的职责是展示，不要抢模型和控制器的活。

7. **<u>任何数据结构的构造或初始化，都应指定大小，避免数据结构无限增长吃光内存。</u>**

8. **<u>及时清理不再使用的代码段或配置信息。</u>**
说明：对于垃圾代码或过时配置，坚决清理干净，避免程序过度臃肿，代码冗余。

正例：对于暂时被注释掉，后续可能恢复使用的代码片断，在注释代码上方，统一规定使用三个斜杠(///)来说明注释掉代码的理由。

## 二、异常日志

### （一）异常处理

1. **<u>Java 类库中定义的一类 RuntimeException 可以通过预先检查进行规避，而不应该通过 catch 来处理。</u>**

  比如： IndexOutOfBoundsException ， NullPointerException 等等。
  说明：无法通过预检查的异常除外，如在解析一个外部传来的字符串形式数字时，通过 catch
  NumberFormatException 来实现。
  正例： if (obj != null) {...}
  反例： try { obj.method() } catch (NullPointerException e) {...}

2. **<u>异常不要用来做流程控制，条件控制，因为异常的处理效率比条件分支低。</u>**

3. **<u>对大段代码进行 try - catch ，这是不负责任的表现。 catch 时请分清稳定代码和非稳定代码，稳定代码指的是无论如何不会出错的代码。对于非稳定代码的 catch 尽可能进行区分异常类型，再做对应的异常处理。</u>**

4. **<u>捕获异常是为了处理它，不要捕获了却什么都不处理而抛弃之，如果不想处理它，请将该异常抛给它的调用者。最外层的业务使用者，必须处理异常，将其转化为用户可以理解的内容。</u>**

5. **<u>有 try 块放到了事务代码中， catch 异常后，如果需要回滚事务，一定要注意手动回滚事务。</u>**

6.  **<u>finally 块必须对资源对象、流对象进行关闭，有异常也要做 try - catch 。</u>**
  说明：如果 JDK 7 及以上，可以使用 try - with - resources 方式。

7. **<u>不能在 finally 块中使用 return ， finally 块中的 return 返回后方法结束执行，不会再执行 try 块中的 return 语句。</u>**

8. **<u>捕获异常与抛异常，必须是完全匹配，或者捕获异常是抛异常的父类。</u>**
  说明：如果预期对方抛的是绣球，实际接到的是铅球，就会产生意外情况。

9. **<u>[推荐]方法的返回值可以为 null ，不强制返回空集合，或者空对象等，必须添加注释充分说明什么情况下会返回 null 值。调用方需要进行 null 判断防止 NPE 问题。</u>**
  说明：本手册明确防止 NPE 是调用者的责任。即使被调用方法返回空集合或者空对象，对调用者来说，也并非高枕无忧，必须考虑到远程调用失败、序列化失败、运行时异常等场景返回null 的情况。

10. **<u>[推荐]防止 NPE ，是程序员的基本修养，注意 NPE 产生的场景：</u>**

    > NPE **NullPointerException**

    1 ） 返回类型为基本数据类型，return 包装数据类型的对象时，自动拆箱有可能产生 NPE。
    反例： public int f() { return Integer 对象}， 如果为 null ，自动解箱抛 NPE 。

    2 ） 数据库的查询结果可能为 null 。
    3 ） 集合里的元素即使 isNotEmpty ，取出的数据元素也可能为 null 。
    4 ） 远程调用返回对象时，一律要求进行空指针判断，防止 NPE 。
    5 ） 对于 Session 中获取的数据，建议 NPE 检查，避免空指针。
    6 ） 级联调用 obj . getA() . getB() . getC()； 一连串调用，易产生 NPE 。
    正例：使用 JDK8 的 Optional 类来防止 NPE 问题。

11. **<u>[推荐]定义时区分 unchecked / checked 异常，避免直接抛出 new RuntimeException() ，更不允许抛出 Exception 或者 Throwable ，应使用有业务含义的自定义异常。推荐业界已定义过的自定义异常，如： DAOException / ServiceException 等。</u>**
12. **<u>在代码中使用“抛异常”还是“返回错误码”，对于公司外的 http / api 开放接口必须使用“错误码” ； 而应用内部推荐异常抛出 ； 跨应用间 RPC 调用优先考虑使用 Result 方式，封装 isSuccess()方法 、“错误码”、“错误简短信息”。</u>**
说明：关于 RPC 方法返回方式使用 Result 方式的理由：
1 ） 使用抛异常返回方式，调用方如果没有捕获到就会产生运行时错误。
2 ） 如果不加栈信息，只是 new 自定义异常，加入自己的理解的 error message ，对于调用
端解决问题的帮助不会太多。如果加了栈信息，在频繁调用出错的情况下，数据序列化和传输
的性能损耗也是问题。
13. **<u>避免出现重复的代码 （Don ’ t Repeat Yourself） ，即 DRY 原则。</u>**
说明：随意复制和粘贴代码，必然会导致代码的重复，在以后需要修改时，需要修改所有的副
本，容易遗漏。必要时抽取共性方法，或者抽象公共类，甚至是组件化。
正例：一个类中有多个 public 方法，都需要进行数行相同的参数校验操作，这个时候请抽取：
private boolean checkParam(DTO dto) {...}

### （二）日志规约

1. **<u>应用中不可直接使用日志系统 （Log 4 j 、 Logback） 中的 API ，而应依赖使用日志框架SLF 4 J 中的 API ，使用门面模式的日志框架，有利于维护和各个类的日志处理方式统一。</u>**
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
private static final Logger logger = LoggerFactory.getLogger(Abc.class);

2. **<u>日志文件推荐至少保存 15 天，因为有些异常具备以“周”为频次发生的特点。</u>**

3. **<u>应用中的扩展日志 （ 如打点、临时监控、访问日志等 ） 命名方式：</u>**
  appName _ logType _ logName . log 。 logType :日志类型，推荐分类有stats / desc / monitor / visit 等 ；logName :日志描述。这种命名的好处：通过文件名就可知道日志文件属于什么应用，什么类型，什么目的，也有利于归类查找。

  正例： mppserver 应用中单独监控时区转换异常，如：
  mppserver _ monitor _ timeZoneConvert . log
  说明：推荐对日志进行分类，如将错误日志和业务日志分开存放，便于开发人员查看，也便于通过日志对系统进行及时监控。

4. **<u>对 trace / debug / info 级别的日志输出，必须使用条件输出形式或者使用占位符的方式。</u>**
说明： logger . debug( " Processing trade with id : " + id + " and symbol : " + symbol);
如果日志级别是 warn ，上述日志不会打印，但是会执行字符串拼接操作，如果 symbol 是对象，会执行 toString() 方法，浪费了系统资源，执行了上述操作，最终日志却没有打印。
正例： （ 条件 ）
if (logger.isDebugEnabled()) {
logger.debug("Processing trade with id: " + id + " and symbol: " + symbol);
}
正例： （ 占位符 ）
logger.debug("Processing trade with id: {} and symbol : {} ", id, symbol);
5. **<u>避免重复打印日志，浪费磁盘空间，务必在 log 4 j . xml 中设置 additivity = false 。</u>**
正例： <logger name="com.taobao.dubbo.config" additivity="false">
6. **<u>异常信息应该包括两类信息：案发现场信息和异常堆栈信息。如果不处理，那么通过关键字 throws 往上抛出。</u>**
正例： logger.error(各类参数或者对象 toString + "_" + e.getMessage(), e);
7. **<u>[推荐]谨慎地记录日志。生产环境禁止输出 debug 日志 ； 有选择地输出 info 日志 ； 如果使用 warn 来记录刚上线时的业务行为信息，一定要注意日志输出量的问题，避免把服务器磁盘撑爆，并记得及时删除这些观察日志。</u>**
说明：大量地输出无效日志，不利于系统性能提升，也不利于快速定位错误点。记录日志时请
思考：这些日志真的有人看吗？看到这条日志你能做什么？能不能给问题排查带来好处？
8. **<u>可以使用 warn 日志级别来记录用户输入参数错误的情况，避免用户投诉时，无所适从。注意日志输出的级别， error 级别只记录系统逻辑出错、异常等重要的错误信息。如非必要，请不要在此场景打出 error 级别。</u>**

## 三、单元测试

1. **<u>好的单元测试必须遵守 AIR 原则。</u>**
  说明：单元测试在线上运行时，感觉像空气 （AIR） 一样并不存在，但在测试质量的保障上，却是非常关键的。好的单元测试宏观上来说，具有自动化、独立性、可重复执行的特点。
  A： Automatic （自动化）
  I： Independent （独立性）

  R： Repeatable （可重复）

2. **<u>单元测试应该是全自动执行的，并且非交互式的。</u>**测试框架通常是定期执行的，执行过程必须完全自动化才有意义。输出结果需要人工检查的测试不是一个好的单元测试。单元测试中不准使用 System.out 来进行人肉验证，必须使用 assert 来验证。

3. **<u>保持单元测试的独立性。</u>**为了保证单元测试稳定可靠且便于维护，单元测试用例之间决不能互相调用，也不能依赖执行的先后次序。
  反例： method2 需要依赖 method1 的执行，将执行结果做为 method2 的输入。

4. **<u>单元测试是可以重复执行的，不能受到外界环境的影响。</u>**
  说明：单元测试通常会被放到持续集成中，每次有代码 check in 时单元测试都会被执行。如果单测对外部环境（网络、服务、中间件等）有依赖，容易导致持续集成机制的不可用。
  正例：为了不受外界环境影响，要求设计代码时就把 SUT 的依赖改成注入，在测试时用 spring这样的 DI 框架注入一个本地（内存）实现或者 Mock 实现。

5. **<u>对于单元测试，要保证测试粒度足够小，有助于精确定位问题。</u>**单测粒度至多是类级别，一般是方法级别。
  说明：只有测试粒度小才能在出错时尽快定位到出错位置。单测不负责检查跨类或者跨系统的交互逻辑，那是集成测试的领域。

6. **<u>核心业务、核心应用、核心模块的增量代码确保单元测试通过。</u>**
  说明：新增代码及时补充单元测试，如果新增代码影响了原有单元测试，请及时修正。

7. **<u>单元测试代码必须写在如下工程目录： src/test/java ，不允许写在业务代码目录下。</u>**
  说明：源码构建时会跳过此目录，而单元测试框架默认是扫描此目录。

8. **<u>[推荐]编写单元测试代码遵守 BCDE 原则，以保证被测试模块的交付质量。</u>**
   B： Border ，边界值测试，包括循环边界、特殊取值、特殊时间点、数据顺序等。
   C： Correct ，正确的输入，并得到预期的结果。
   D： Design ，与设计文档相结合，来编写单元测试。
   E： Error ，强制错误信息输入（如：非法数据、异常流程、非业务允许输入等），并得
   到预期的结果。
9. **<u>[推荐]对于数据库相关的查询，更新，删除等操作，不能假设数据库里的数据是存在的，或者直接操作数据库把数据插入进去，请使用程序插入或者导入数据的方式来准备数据。</u>**
   反例：删除某一行数据的单元测试，在数据库中，先直接手动增加一行作为删除目标，但是这一行新增数据并不符合业务插入规则，导致测试结果异常。
10. **<u>和数据库相关的单元测试，可以设定自动回滚机制，不给数据库造成脏数据。或者对单元测试产生的数据有明确的前后缀标识。</u>**
    正例：在 RDC 内部单元测试中，使用 RDC_UNIT_TEST_ 的前缀标识数据。
11. **<u>对于不可测的代码建议做必要的重构，使代码变得可测，避免为了达到测试要求而书写不规范测试代码。</u>**

## 四、安全规约

1. **<u>隶属于用户个人的页面或者功能必须进行权限控制校验。</u>**
   说明：防止没有做水平权限校验就可随意访问、修改、删除别人的数据，比如查看他人的私信内容、修改他人的订单。

2. **<u>用户敏感数据禁止直接展示，必须对展示数据进行脱敏。</u>**
   说明：查看个人手机号码会显示成:158****9119，隐藏中间 4 位，防止隐私泄露。

3. **<u>用户输入的 SQL 参数严格使用参数绑定或者 METADATA 字段值限定，防止 SQL 注入，禁止字符串拼接 SQL 访问数据库。</u>**

4. 用户请求传入的任何参数必须做有效性验证。
   说明：忽略参数校验可能导致：
   page size 过大导致内存溢出
   恶意 order by 导致数据库慢查询
   任意重定向
   SQL 注入
   反序列化注入
   正则输入源串拒绝服务 ReDoS

   > *ReDoS*(Regular expression Denial of Service) 正则表达式拒绝服务*攻击*

5. **<u>禁止向 HTML 页面输出未经安全过滤或未正确转义的用户数据。</u>**

6. **<u>表单、 AJAX 提交必须执行 CSRF 安全过滤。</u>**
   说明： CSRF(Cross - site request forgery) 跨站请求伪造是一类常见编程漏洞。对于存在CSRF 漏洞的应用/网站，攻击者可以事先构造好 URL ，只要受害者用户一访问，后台便在用户不知情情况下对数据库中用户参数进行相应修改。

7. **<u>在使用平台资源，譬如短信、邮件、电话、下单、支付，必须实现正确的防重放限制，如数量限制、疲劳度控制、验证码校验，避免被滥刷、资损。</u>**
   说明：如注册时发送验证码到手机，如果没有限制次数和频率，那么可以利用此功能骚扰到其它用户，并造成短信平台资源浪费。

8. **<u>[推荐]发贴、评论、发送即时消息等用户生成内容的场景必须实现防刷、文本内容违禁词过滤等风控策略。</u>**

## 五、MySQL数据库

### （一）建表规范

1. **<u>表名、字段名必须使用小写字母或数字 ， 禁止出现数字开头，禁止两个下划线中间只出现数字。数据库字段名的修改代价很大，因为无法进行预发布，所以字段名称需要慎重考虑。</u>**
   说明：MySQL 在 Windows 下不区分大小写，但在 Linux 下默认是区分大小写。因此，数据库
   名、表名、字段名，都不允许出现任何大写字母，避免节外生枝。
   正例： aliyun _ admin ， rdc _ config ， level 3_ name
   反例： AliyunAdmin ， rdcConfig ， level _3_ name

2. **<u>表名不使用复数名词。</u>**
   说明：表名应该仅仅表示表里面的实体内容，不应该表示实体数量，对应于 DO 类名也是单数
   形式，符合表达习惯。

3. **<u>禁用保留字，如 desc 、 range 、 match 、 delayed 等，请参考 MySQL 官方保留字。</u>**

   > [MySQL 保留字](https://dev.mysql.com/doc/refman/5.7/en/keywords.html)

4. **<u>varchar 是可变长字符串，不预先分配存储空间，长度不要超过 5000，如果存储长度大于此值，定义字段类型为 text ，独立出来一张表，用主键来对应，避免影响其它字段索引效率。</u>**

### （二）索引规约

1. **<u>业务上具有唯一特性的字段，即使是多个字段的组合，也必须建成唯一索引。</u>**

   不要以为唯一索引影响了 insert 速度，这个速度损耗可以忽略，但提高查找速度是明
   显的 ； 另外，即使在应用层做了非常完善的校验控制，只要没有唯一索引，根据墨菲定律，必
   然有脏数据产生。

2. **<u>超过三个表禁止 join 。</u>**需要 join 的字段，数据类型必须绝对一致 ； 多表关联查询时，保证被关联的字段需要有索引。
   说明：即使双表 join 也要注意表索引、 SQL 性能。

3. **<u>在 varchar 字段上建立索引时，必须指定索引长度，没必要对全字段建立索引，根据实际文本区分度决定索引长度即可。</u>**

   索引的长度与区分度是一对矛盾体，一般对字符串类型数据，长度为 20 的索引，区分度会高达 90%以上，可以使用 count(distinct left( 列名, 索引长度 )) / count( * ) 的区分度来确定。

4. **<u>如果有 order by 的场景，请注意利用索引的有序性。</u>** order by 最后的字段是组合
   索引的一部分，并且放在索引组合顺序的最后，避免出现 file _ sort 的情况，影响查询性能。
   正例： where a =? and b =? order by c; 索引： a _ b _ c
   反例：索引中有范围查找，那么索引有序性无法利用，如： WHERE a >10 ORDER BY b; 索引
   a _ b 无法排序。

### （三）SQL语句

1. **<u>不要使用 count( 列名 ) 或 count( 常量 ) 来替代 count( * )</u>** ， count( * ) 是 SQL 92 定义的
   标准统计行数的语法，跟数据库无关，跟 NULL 和非 NULL 无关。
   说明： count( * ) 会统计值为 NULL 的行，而 count( 列名 ) 不会统计此列为 NULL 值的行。

   count(distinct col) 计算该列除 NULL 之外的不重复行数，注意 count(distinct
   col 1, col 2 ) 如果其中一列全为 NULL ，那么即使另一列有不同的值，也返回为 0。

2. **<u>当某一列的值全是 NULL 时， count(col) 的返回结果为 0，但 sum(col) 的返回结果为NULL ，因此使用 sum() 时需注意 NPE 问题。</u>**
   正例：可以使用如下方式来避免 sum 的 NPE 问题： SELECT IF(ISNULL(SUM(g)) ,0, SUM(g))
   FROM table;

3. **<u>使用 ISNULL() 来判断是否为 NULL 值。</u>**
   说明： NULL 与任何值的直接比较都为 NULL。
   1 ） NULL<>NULL 的返回结果是 NULL ，而不是 false 。
   2 ） NULL=NULL 的返回结果是 NULL ，而不是 true 。
   3 ） NULL<>1 的返回结果是 NULL ，而不是 true 。

4. **<u>在代码中写分页查询逻辑时，若 count 为 0 应直接返回，避免执行后面的分页语句。</u>**

5. **<u>如果有全球化需要，所有的字符存储与表示，均以 utf -8 编码，注意字符统计函数的区别。</u>**

6. **<u>TRUNCATE TABLE 比 DELETE 速度快，且使用的系统和事务日志资源少，但 TRUNCATE无事务且不触发 trigger ，有可能造成事故，故不建议在开发代码中使用此语句。</u>**
   说明： TRUNCATE TABLE 在功能上与不带 WHERE 子句的 DELETE 语句相同。

### （四）ORM映射

1. **<u>在表查询中，一律不要使用 * 作为查询的字段列表，需要哪些字段必须明确写明。</u>**
   说明：

   1 ） 增加查询分析器解析成本。

   2 ） 增减字段容易与 resultMap 配置不一致。

2. **<u>不要用 resultClass 当返回参数，即使所有类属性名与数据库字段一一对应，也需要定义 ； 反过来，每一个表也必然有一个与之对应。</u>**
   说明：配置映射关系，使字段与 DO 类解耦，方便维护。
3. **<u>sql. xml 配置参数使用：#{}，# param # 不要使用${} 此种方式容易出现 SQL 注入。</u>**
4. **<u>iBATIS 自带的 queryForList(String statementName , int start , int size) 不推荐使用。</u>**
   说明：其实现方式是在数据库取到 statementName 对应的 SQL 语句的所有记录，再通过 subList
   取 start , size 的子集合。
   正例： Map<String, Object> map = new HashMap<String, Object>();
   map.put("start", start);
   map.put("size", size);
5. **<u>不允许直接拿 HashMap 与 Hashtable 作为查询结果集的输出。</u>**
   说明： resultClass=”Hashtable” ，会置入字段名和属性值，但是值的类型不可控。
6. **<u>[推荐]不要写一个大而全的数据更新接口。传入为 POJO 类，不管是不是自己的目标更新字段，都进行 update table set c1=value1,c2=value2,c3=value3; 这是不对的。执行 SQL时，不要更新无改动的字段，一是易出错 ； 二是效率低 ； 三是增加 binlog 存储。</u>**

